﻿using System;
using System.Data;

namespace DBShardTools.Core
{
    public abstract class PersisterBase : IPersister
    {
        public virtual string ConnectionString
        {
            get
            {
                return GetConnectionStringBuilder().BuildConnectionString(this);
            }
        }
        public virtual int ExecuteNoQuery(string commandText)
        {
            int result = 0;
            IDbConnection conn;
            IDbCommand comm;
            using (conn = GetConnection())
            {
                comm = conn.CreateCommand();
                conn.Open();
                comm.CommandText = commandText;
                try
                {
                    result = comm.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            return result;
        }

        public int ExecuteNoQueryBulky(string[] allCommandText)
        {
            int result = 0;
            IDbConnection conn;
            IDbCommand comm;
            IDbTransaction tran;
            using (conn = GetConnection())
            {
                comm = conn.CreateCommand();
                conn.Open();
                tran = conn.BeginTransaction();
                comm.Transaction = tran;
                try
                {
                    foreach (string commandText in allCommandText)
                    {
                        comm.CommandText = commandText;
                        result += comm.ExecuteNonQuery();
                    }
                    tran.Commit();
                }
                catch(Exception ex)
                {
                    tran.Rollback();
                    throw ex;
                }
            }
            return result;
        }

        public virtual bool TestConnection()
        {
            IDbConnection conn = GetConnection();
            try
            {
                conn.Open();
                return true;
            }
            catch(Exception ex)
            {
                return false;
            }
            finally
            {
                conn.Close();
            }

        }

        public abstract string GetCreateTableDdl(string tableName);
        public abstract string GetAlterTableDdl(string tableName);
        public abstract string GetDropTableDdl(string tableName);
        public abstract string[] GetTables();

        private string _host;
        public virtual string Host
        {
            get
            {
                if (string.IsNullOrEmpty(_host))
                    _host = "localhost";
                return _host;
            }
            set { _host = value; }
        }
        public abstract int Port { get; set; }
        public abstract string UserName { get; set; }
        public virtual string Password { get; set; }
        public virtual string Database{get;set;}
        public virtual IDbConnection Connection { get; protected set; }
        public abstract IDbConnection GetConnection();
        public abstract IConnectionStringBuilder GetConnectionStringBuilder();
        public abstract string Name { get; }
        public abstract string Description { get; }
    }
}
