﻿using System;
using System.Collections.Generic;

namespace DBShardTools.Core
{
    public class ShardService : IShardService
    {
        private readonly IPersister _persister;
        private readonly INameStrategy _nameStrategy;
        public ShardService(INameStrategy nameStrategy,IPersister persister)
        {
            _nameStrategy = nameStrategy;
            _persister = persister;
        }

        public bool ShardTable(string tableDdl, string format, bool useTran,Action<string> callback)
        {
            string tableName = Utils.ParseTableNameFromCreateTableDdl(tableDdl);
            if (string.IsNullOrEmpty(tableName))
            {
                callback("Invalid table name.");
                return false;
            }
            int result = 0;
            if (_nameStrategy != null)
            {
                string[] tableNumbers = _nameStrategy.GetTableNumbers();

                if (!useTran)
                {
                    foreach (string tableNumber in tableNumbers)
                    {
                        string shardTableName = string.Format(format, tableNumber);
                        string commandText = Utils.ReplaceTableNameOfCreateTableDdl(tableDdl, shardTableName);
                        try
                        {
                            _persister.ExecuteNoQuery(commandText);
                            result++;
                            callback(string.Format("Create table {0} completed.", shardTableName));
                        }
                        catch (Exception ex)
                        {
                            callback(ex.Message);
                        }
                    }
                }
                else
                {
                    List<string> allCommandText = new List<string>();
                    foreach (string tableNumber in tableNumbers)
                    {
                        string shardTableName = string.Format(format, tableNumber);
                        string commandText = Utils.ReplaceTableNameOfCreateTableDdl(tableDdl, shardTableName);
                        allCommandText.Add(commandText);
                    }
                    try
                    {
                        _persister.ExecuteNoQueryBulky(allCommandText.ToArray());
                        result = tableNumbers.Length;
                    }
                    catch (Exception ex)
                    {
                        callback(ex.Message);
                    }
                }
                callback(string.Format("Total {0} tables create completed.", result));
            }

            return true;
        }

        public bool AlterTable(string tableDdl, string format, bool useTran, Action<string> callback)
        {
            string tableName = Utils.ParseTableNameFromAlterTableDdl(tableDdl);
            if (string.IsNullOrEmpty(tableName))
            {
                callback("Invalid table name.");
                return false;
            }
            int result = 0;
            if (_nameStrategy != null)
            {
                string[] tableNumbers = _nameStrategy.GetTableNumbers();

                if (!useTran)
                {
                    foreach (string tableNumber in tableNumbers)
                    {
                        string shardTableName = string.Format(format, tableNumber);
                        string commandText = Utils.ReplaceTableNameOfAlterTableDdl(tableDdl, shardTableName);
                        try
                        {
                            _persister.ExecuteNoQuery(commandText);
                            result++;
                            callback(string.Format("Alter table {0} completed.", shardTableName));
                        }
                        catch (Exception ex)
                        {
                            callback(ex.Message);
                        }
                    }
                }
                else
                {
                    List<string> allCommandText = new List<string>();
                    foreach (string tableNumber in tableNumbers)
                    {
                        string shardTableName = string.Format(format, tableNumber);
                        string commandText = Utils.ReplaceTableNameOfAlterTableDdl(tableDdl, shardTableName);
                        allCommandText.Add(commandText);
                    }
                    try
                    {
                        _persister.ExecuteNoQueryBulky(allCommandText.ToArray());
                        result = tableNumbers.Length;
                    }
                    catch (Exception ex)
                    {
                        callback(ex.Message);
                    }
                }
                callback(string.Format("Total {0} tables alter completed.", result));
            }

            return true;
        }

        public bool DropTable(string tableDdl, string format, bool useTran, Action<string> callback)
        {
            string tableName = Utils.ParseTableNameFromDropTableDdl(tableDdl);
            if (string.IsNullOrEmpty(tableName))
            {
                callback("Invalid table name.");
                return false;
            }
            int result = 0;
            if (_nameStrategy != null)
            {
                string[] tableNumbers = _nameStrategy.GetTableNumbers();

                if (!useTran)
                {
                    foreach (string tableNumber in tableNumbers)
                    {
                        string shardTableName = string.Format(format, tableNumber);
                        string commandText = Utils.ReplaceTableNameOfDropTableDdl(tableDdl, shardTableName);
                        try
                        {
                            _persister.ExecuteNoQuery(commandText);
                            result++;
                            callback(string.Format("Drop table {0} completed.", shardTableName));
                        }
                        catch (Exception ex)
                        {
                            callback(ex.Message);
                        }
                    }
                }
                else
                {
                    List<string> allCommandText = new List<string>();
                    foreach (string tableNumber in tableNumbers)
                    {
                        string shardTableName = string.Format(format, tableNumber);
                        string commandText = Utils.ReplaceTableNameOfDropTableDdl(tableDdl, shardTableName);
                        allCommandText.Add(commandText);
                    }
                    try
                    {
                        _persister.ExecuteNoQueryBulky(allCommandText.ToArray());
                        result = tableNumbers.Length;
                    }
                    catch (Exception ex)
                    {
                        callback(ex.Message);
                    }
                }
                callback(string.Format("Total {0} tables drop completed.", result));
            }

            return true;
        }
    }
}
