﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Windows.Forms;
using DBShardTools.Core;
using Microsoft.Practices.Unity;

namespace DBShardTools.UI
{
    public sealed partial class ShardTableForm : Form
    {
        private  IShardService _shardService;
        private readonly IUnityContainer _container;
        private OperateType _operateType = OperateType.NotDefined;
        private bool _useTransaction = false;
        public ShardTableForm(IUnityContainer container,IShardService shardService)
        {
            InitializeComponent();


            _container = container;
            _shardService = shardService;
            chkbUseTran.Checked = false;

        }

        private void BtnConfirmClick(object sender, EventArgs e)
        {
            switch(_operateType)
            {
                case OperateType.CreateShardTable:
                    {

                        ProcessShardTable(_useTransaction);
                    }
                    break;
                case OperateType.AlterShardTable:
                    {
                        ProcessAlterTable(_useTransaction);
                    }
                    break;
                case OperateType.DropShardTable:
                    {
                        ProcessDropTable(_useTransaction);
                    }
                    break;
                default:
                    break;

            }
            
        }
        private void ProcessShardTable(bool useTran)
        {
            new Thread(() =>
            {
                SetConfirmButtonState(false);
                _shardService.ShardTable(txtDdl.Text.Trim(), txtFormat.Text.Trim(), useTran, ProcessResultCallback);
                SetConfirmButtonState(true);
            }).Start();

        }
        private void ProcessAlterTable(bool useTran)
        {
            new Thread(() =>
            {
                SetConfirmButtonState(false);
                _shardService.AlterTable(txtDdl.Text.Trim(), txtFormat.Text.Trim(), useTran, ProcessResultCallback);
                SetConfirmButtonState(true);
            }).Start();

        }
        private void ProcessDropTable(bool useTran)
        {
            new Thread(() =>
            {
                SetConfirmButtonState(false);
                _shardService.DropTable(txtDdl.Text.Trim(), txtFormat.Text.Trim(), useTran, ProcessResultCallback);
                SetConfirmButtonState(true);
            }).Start();

        }
        private void ProcessResultCallback(string result)
        {
            if (lstProcessResult.InvokeRequired)
            {
                this.Invoke(new Action(() =>
                {
                    lstProcessResult.Items.Add(result);
                }));
            }
            else
                lstProcessResult.Items.Add(result);
        }
        private void SetConfirmButtonState(bool enabled)
        {
            if (btnConfirm.InvokeRequired)
            {
                this.Invoke(new Action(() =>
                {
                    btnConfirm.Enabled = enabled;
                }));
            }
            else
                btnConfirm.Enabled = enabled;
        }
        private void SetShardFormat(string tableName)
        {
            string format = string.Empty;
            if (!string.IsNullOrEmpty(tableName))
                format = tableName + "_{0}";
            txtFormat.Text = format;
        }

        private void BtnClearResultClick(object sender, EventArgs e)
        {
            lstProcessResult.Items.Clear();
        }
        public void ConfigChangedCallback()
        {
            _shardService = _container.Resolve<IShardService>();
            
        }
        private void DataBindMainTableList()
        {
            List<string> maintableList = new List<string>();
            maintableList.AddRange(_container.Resolve<IPersister>().GetTables());
            ddlMainTable.DataSource = maintableList;

        }

        private void ChkbSelectMaintableCheckStateChanged(object sender, EventArgs e)
        {
            if (chkbSelectMaintable.Checked)
            {
                ddlMainTable.DropDownStyle = ComboBoxStyle.DropDownList;
                try
                {
                    DataBindMainTableList();
                }
                catch(NotImplementedException ex)
                {
                    MessageBox.Show("该数据库类型不支持此操作，请手工输入！", "系统消息", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    chkbSelectMaintable.Checked = false;
                }
            }
            else
            {
                ddlMainTable.DropDownStyle = ComboBoxStyle.Simple;
            }
        }

        private void DdlMainTableTextChanged(object sender, EventArgs e)
        {
            SetShardFormat(ddlMainTable.Text);
        }

        private void RdoOperateTypeCheckedChanged(object sender, EventArgs e)
        {
            RadioButton rboCurSelect = (RadioButton)sender;
            if (rboCurSelect.Checked)
            {
                _operateType = (OperateType)Enum.Parse(typeof(OperateType), rboCurSelect.Tag.ToString());
                switch (_operateType)
                {
                    case OperateType.CreateShardTable:
                        {
                            try
                            {
                                txtDdl.Text = _container.Resolve<IPersister>().GetCreateTableDdl(ddlMainTable.Text.Trim());
                            }
                            catch (NotImplementedException ex)
                            {
                                MessageBox.Show("该数据库类型不支持此操作，请手工输入！", "系统消息", MessageBoxButtons.OK, MessageBoxIcon.Error);

                            }
                        }
                        break;
                    case OperateType.AlterShardTable:
                        {
                            try
                            {
                                txtDdl.Text = _container.Resolve<IPersister>().GetAlterTableDdl(ddlMainTable.Text.Trim());
                            }
                            catch (NotImplementedException ex)
                            {
                                MessageBox.Show("该数据库类型不支持此操作，请手工输入！", "系统消息", MessageBoxButtons.OK, MessageBoxIcon.Error);

                            }
                        }
                        break;
                    case OperateType.DropShardTable:
                        {
                            try
                            {
                                txtDdl.Text = _container.Resolve<IPersister>().GetDropTableDdl(ddlMainTable.Text.Trim());
                            }
                            catch (NotImplementedException ex)
                            {
                                MessageBox.Show("该数据库类型不支持此操作，请手工输入！", "系统消息", MessageBoxButtons.OK, MessageBoxIcon.Error);

                            }
                        }
                        break;
                    default:
                        break;

                }
            }
        }

        private void ChkbUseTranCheckedChanged(object sender, EventArgs e)
        {
            _useTransaction = chkbUseTran.Checked;
        }
    }
}
